Module: RedisQueuedLocks
- Extended by:
- Debugger::Interface
- Defined in:
- lib/redis_queued_locks.rb,
lib/redis_queued_locks/errors.rb,
lib/redis_queued_locks/version.rb
Overview
NOTE:
the one-line definition style (like "Error = Class.new(::StandardError)")
is not used cuz that style can not be correctly defined via RBS types: we can not
identify the superclass of our class for a constant;
Defined Under Namespace
Modules: Acquirer, Debugger, Instrument, Logging, Resource, Utilities Classes: ArgumentError, Client, Config, ConfigError, ConfigNotFoundError, ConfigValidationError, ConflictLockObtainError, Data, Error, LockAcquirementIntermediateTimeoutError, LockAcquirementRetryLimitError, LockAcquirementTimeoutError, LockAlreadyObtainedError, Swarm, SwarmArgumentError, SwarmError, TimedLockIntermediateTimeoutError, TimedLockTimeoutError
Constant Summary collapse
- VERSION =
'1.16.2'